// Generated by CoffeeScript 1.6.3
var Client, async, helpers,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

require('colors');

async = require('async');

helpers = require('./helpers');

Client = require("request-json").JsonClient;

Client.prototype.configure = function(url, password, callback) {
  this.host = url;
  return this.post("login", {
    password: password
  }, function(err, res, body) {
    if (err || res.statusCode !== 200) {
      return console.log("Cannot get authenticated".red);
    } else {
      return callback();
    }
  });
};

exports.ApplicationManager = (function() {
  function ApplicationManager() {
    this.isInstalled = __bind(this.isInstalled, this);
  }

  ApplicationManager.prototype.client = new Client("");

  ApplicationManager.prototype.checkError = function(err, res, body, code, msg, callback) {
    if (err || res.statusCode !== code) {
      if (err != null) {
        console.log(err.red);
      }
      console.log(msg);
      if (body != null) {
        if (body.msg != null) {
          return console.log(body.msg);
        } else {
          return console.log(body);
        }
      }
    } else {
      return callback();
    }
  };

  ApplicationManager.prototype.updateApp = function(app, url, password, callback) {
    var _this = this;
    console.log("Update " + app + "...");
    return this.client.configure(url, password, function() {
      var path;
      path = "api/applications/" + app + "/update";
      return _this.client.put(path, {}, function(err, res, body) {
        var output;
        output = 'Update failed.'.red;
        return _this.checkError(err, res, body, 200, output, callback);
      });
    });
  };

  ApplicationManager.prototype.installApp = function(app, url, repoUrl, password, callback) {
    var _this = this;
    console.log("Install started for " + app + "...");
    return this.client.configure(url, password, function() {
      var app_descriptor, path;
      app_descriptor = {
        name: app,
        git: repoUrl
      };
      path = "api/applications/install";
      return _this.client.post(path, app_descriptor, function(err, res, body) {
        var output;
        output = 'Install failed.'.red;
        return _this.checkError(err, res, body, 201, output, callback);
      });
    });
  };

  ApplicationManager.prototype.uninstallApp = function(app, url, password, callback) {
    var _this = this;
    console.log("Uninstall started for " + app + "...");
    return this.client.configure(url, password, function() {
      var path;
      path = "api/applications/" + app + "/uninstall";
      return _this.client.del(path, function(err, res, body) {
        var output;
        output = 'Uninstall failed.'.red;
        return _this.checkError(err, res, body, 200, output, callback);
      });
    });
  };

  ApplicationManager.prototype.checkStatus = function(url, password, callback) {
    var checkApp, checkStatus,
      _this = this;
    checkApp = function(app) {
      return function(next) {
        var path;
        if (app !== "home" && app !== "proxy") {
          path = "apps/" + app + "/";
        } else {
          path = "";
        }
        return _this.client.get(path, function(err, res) {
          if (err || res.statusCode !== 200) {
            console.log(("" + app + ": ") + "down".red);
          } else {
            console.log(("" + app + ": ") + "up".green);
          }
          return next();
        });
      };
    };
    checkStatus = function() {
      return async.series([checkApp("home"), checkApp("proxy", "routes")], function() {
        return _this.client.get("api/applications/", function(err, res, apps) {
          var app, funcs, _i, _len, _ref;
          if (err) {
            return console.log(err.red);
          } else {
            funcs = [];
            if ((apps != null) && typeof apps === "object") {
              _ref = apps.rows;
              for (_i = 0, _len = _ref.length; _i < _len; _i++) {
                app = _ref[_i];
                funcs.push(checkApp(app.name));
              }
              return async.series(funcs, function() {
                return callback();
              });
            }
          }
        });
      });
    };
    return this.client.configure(url, password, checkStatus);
  };

  ApplicationManager.prototype.isInstalled = function(app, url, password, callback) {
    var _this = this;
    return this.client.configure(url, password, function() {
      return _this.client.get("apps/" + (app.toLowerCase()) + "/", function(err, res, body) {
        if (body === "app unknown") {
          return callback(null, false);
        } else if (err) {
          return callback(err, false);
        } else {
          return callback(null, true);
        }
      });
    });
  };

  return ApplicationManager;

})();
