// Generated by CoffeeScript 1.6.3
var ApplicationManager, Client, ProjectManager, RepoManager, VagrantManager, appData, applicationManager, client, haltOption, helpers, path, program, projectManager, repoManager, vagrantManager;

require('colors');

program = require('commander');

path = require('path');

Client = require("request-json").JsonClient;

Client.prototype.configure = function(url, password, callback) {
  this.host = url;
  return this.post("login", {
    password: password
  }, function(err, res) {
    if (err || res.statusCode !== 200) {
      return console.log("Cannot get authenticated");
    } else {
      return callback();
    }
  });
};

client = new Client("");

RepoManager = require('./repository').RepoManager;

repoManager = new RepoManager();

ApplicationManager = new require('./application').ApplicationManager;

applicationManager = new ApplicationManager();

ProjectManager = new require('./project').ProjectManager;

projectManager = new ProjectManager();

VagrantManager = new require('./vagrant').VagrantManager;

vagrantManager = new VagrantManager();

helpers = require('./helpers');

appData = require('../package.json');

/* Tasks*/


program.version(appData.version).option('-u, --url <url>', 'set url where lives your Cozy Cloud, default to localhost').option('-g, --github <github>', 'Link new project to github account').option('-c, --coffee', 'Create app template with coffee script files instead of JS files');

program.command("install <app> <repo>").description("Install given application from its repository").action(function(app, repo) {
  return program.password("Cozy password:", function(password) {
    return appManager.installApp(app, program.url, repo, password, function() {
      return console.log(("" + app + " sucessfully installed.").green);
    });
  });
});

program.command("uninstall <app>").description("Uninstall given application").action(function(app) {
  return program.password("Cozy password:", function(password) {
    return appManager.uninstallApp(app, program.url, password, function() {
      return console.log(("" + app + " sucessfully uninstalled.").green);
    });
  });
});

program.command("update <app>").description("Update application (git + npm install) and restart it through haibu").action(function(app) {
  return program.password("Cozy password:", function(password) {
    return appManager.updateApp(app, program.url, password, function() {
      return console.log(("" + app + " sucessfully updated.").green);
    });
  });
});

program.command("status").description("Give current state of cozy platform applications").action(function() {
  return program.password("Cozy password:", function(password) {
    return appManager.checkStatus(program.url, password, function() {
      return console.log("All apps have been checked.".green);
    });
  });
});

program.command("new <appname>").description("Create a new app suited to be deployed on a Cozy Cloud.").action(function(appname) {
  var isCoffee, user,
    _this = this;
  user = program.github;
  isCoffee = program.coffee;
  if (user != null) {
    console.log("Create repo " + appname + " for user " + user + "...");
    return program.password("Github password:", function(password) {
      return program.prompt("Cozy Url:", function(url) {
        return projectManager.newProject(appname, isCoffee, url, user, password, function() {
          console.log("Project creation finished.".green);
          return process.exit(0);
        });
      });
    });
  } else {
    console.log("Create project folder: " + appname);
    return repoManager.createLocalRepo(appname, isCoffee, function() {
      return console.log("Project creation finished.".green);
    });
  }
});

program.command("deploy").description("Push code and deploy app located in current directory " + "to Cozy Cloud url configured in configuration file.").action(function() {
  var config;
  config = require(path.join(process.cwd(), ".cozy_conf.json"));
  return program.password("Cozy password:", function(password) {
    return projectManager.deploy(config, password, function() {
      return console.log(("" + config.cozy.appName + " sucessfully deployed.").green);
    });
  });
});

program.command("dev:init").description("Initialize the current folder to host a virtual machine " + "with Vagrant. This will download the base box file.").action(function() {
  console.log("Initializing the virtual machine in the folder..." + "this may take a while.");
  return vagrantManager.checkIfVagrantIsInstalled(function() {
    return vagrantManager.vagrantBoxAdd(function() {
      return vagrantManager.vagrantInit(function() {
        var msg;
        msg = "The virtual machine has been successfully " + "initialized.";
        return console.log(msg.green);
      });
    });
  });
});

program.command("dev:destroy").description("Destroy the virtual machine. Data will be lost.").action(function() {
  var confirmMessage;
  confirmMessage = "You are about to remove the virtual machine from " + "your computer. All data will be lost and a new " + "import will be required if you want to use the " + "VM again. [y/n]";
  return program.confirm(confirmMessage, function(ok) {
    if (ok) {
      return vagrantManager.vagrantBoxDestroy(function() {
        var msg;
        msg = "The box has been sucessfully destroyed. Use " + "cozy dev:init to be able to use the VM again.";
        console.log(msg.green);
        return process.exit();
      });
    }
  });
});

program.command("dev:start").description("Starts the virtual machine with Vagrant.").action(function() {
  return vagrantManager.checkIfVagrantIsInstalled(function() {
    console.log("Starting the virtual machine...this may take a while.");
    return vagrantManager.vagrantUp(function(code) {
      var msg;
      if (code === 0) {
        msg = "The virtual machine has been successfully " + "started. You can check everything is working " + "by running cozy dev:vm-status.";
        return console.log(msg.green);
      } else {
        msg = "An error occurred while your VMs was starting.";
        return console.log(msg.red);
      }
    });
  });
});

haltOption = "Properly stop the virtual machine instead of simply " + "suspending its execution";

program.command("dev:stop").option("-H, --halt", haltOption).description("Stops the Virtual machine with Vagrant.").action(function() {
  var option,
    _this = this;
  option = this.args[0].halt;
  return vagrantManager.checkIfVagrantIsInstalled(function() {
    var caller;
    console.log("Stopping the virtual machine...this may take a while.");
    if ((option != null) && option) {
      caller = vagrantManager.vagrantHalt;
    } else {
      caller = vagrantManager.vagrantSuspend;
    }
    return caller(function(code) {
      var msg;
      if (code === 0) {
        msg = "The virtual machine has been successfully stopped.";
        return console.log(msg.green);
      } else {
        msg = "An error occurred while your VMs was shutting down.";
        return console.log(msg.red);
      }
    });
  });
});

program.command("dev:vm-status").description("Tells which services of the VM are running and accessible.").action(function() {
  return vagrantManager.checkIfVagrantIsInstalled(function() {
    return vagrantManager.virtualMachineStatus(function(code) {
      var msg;
      if (code === 0) {
        msg = "All the core services are up and running.";
        return console.log(msg.green);
      } else {
        return console.log("One or more services are not running.".red);
      }
    });
  });
});

program.command("dev:light-update").description("Updates the virtual machine with the latest version of " + "the cozy PaaS and core applications").action(function() {
  return vagrantManager.checkIfVagrantIsInstalled(function() {
    return vagrantManager.lightUpdate(function(code) {
      if (code === 0) {
        return console.log("VM updated.".green);
      } else {
        return console.log("An error occurred while updating the VM".red);
      }
    });
  });
});

program.command("*").description("Display error message for an unknown command.").action(function() {
  return console.log('Unknown command, run "cozy --help"' + ' to know the list of available commands.');
});

program.parse(process.argv);
