// Generated by CoffeeScript 1.6.3
var Client, fs, helpers, path;

require('colors');

path = require('path');

fs = require('fs');

Client = require('request-json').JsonClient;

helpers = require('./helpers');

exports.RepoManager = (function() {
  function RepoManager() {}

  RepoManager.prototype.createLocalRepo = function(appname, isCoffee, callback) {
    var appPath, cmds, removeGitFolderCommand, repo;
    cmds = [];
    if (!isCoffee) {
      repo = "https://github.com/mycozycloud/cozy-template.git";
    } else {
      repo = "https://github.com/mycozycloud/cozy-template-coffee.git";
    }
    appPath = "" + (process.cwd()) + "/" + appname;
    cmds.push({
      name: "git",
      args: ['clone', repo, appname],
      opts: {
        cwd: process.cwd()
      }
    });
    cmds.push({
      name: "git",
      args: ['submodule', 'update', '--init', '--recursive'],
      opts: {
        cwd: appPath
      }
    });
    if (helpers.isRunningOnWindows()) {
      removeGitFolderCommand = {
        name: "rmdir",
        args: ['/S', '/Q', '.git'],
        opts: {
          cwd: appPath
        }
      };
    } else {
      removeGitFolderCommand = {
        name: "rm",
        args: ['-rf', '.git'],
        opts: {
          cwd: appPath
        }
      };
    }
    cmds.push(removeGitFolderCommand);
    cmds.push({
      name: "npm",
      args: ['install'],
      opts: {
        cwd: appPath
      }
    });
    cmds.push({
      name: "npm",
      args: ['install'],
      opts: {
        cwd: "" + appPath + "/client"
      }
    });
    console.log("Creating the project structure...");
    return helpers.spawnUntilEmpty(cmds, function(code) {
      var msg;
      if (code === 0) {
        console.log("Project structure created.".green);
      } else {
        msg = "An error occurrend during project structure creation";
        console.log(msg.red);
      }
      return callback();
    });
  };

  RepoManager.prototype.connectRepos = function(user, appname, callback) {
    var appPath, cmds, remoteRepository;
    cmds = [];
    appPath = "" + (process.cwd()) + "/" + appname;
    cmds.push({
      name: "git",
      args: ['init'],
      opts: {
        cwd: appPath
      }
    });
    remoteRepository = "git@github.com:" + user + "/" + appname + ".git";
    cmds.push({
      name: "git",
      args: ['remote', 'add', 'origin', remoteRepository],
      opts: {
        cwd: appPath
      }
    });
    cmds.push({
      name: "git",
      args: ['add', '.'],
      opts: {
        cwd: appPath
      }
    });
    cmds.push({
      name: "git",
      args: ['commit', '-a', '-m', '"First commit."'],
      opts: {
        cwd: appPath
      }
    });
    cmds.push({
      name: "git",
      args: ['push', 'origin', '-u', 'master'],
      opts: {
        cwd: appPath
      }
    });
    return helpers.spawnUntilEmpty(cmds, function() {
      var msg;
      msg = "The project has been successfully linked to a Github " + "repository.";
      console.log(msg.green);
      return callback();
    });
  };

  RepoManager.prototype.createGithubRepo = function(credentials, repo, callback) {
    var client;
    client = new Client('https://api.github.com/');
    client.setBasicAuth(credentials.username, credentials.password);
    return client.post('user/repos', {
      name: repo
    }, function(err, res, body) {
      if (err) {
        console.log("An error occured while creating repository.".red);
        return console.log(err);
      } else if (res.statusCode !== 201) {
        console.log("Cannot create repository on Github.".red);
        return console.log(body);
      } else {
        return callback();
      }
    });
  };

  RepoManager.prototype.saveConfig = function(githubUser, app, url, callback) {
    var data;
    data = "{\n    \"cozy\": {\n        \"appName\": \"" + app + "\",\n        \"url\": \"" + url + "\"\n    },\n    \"github\": {\n        \"user\": \"" + githubUser + "\",\n        \"repoName\": \"" + app + "\"\n    }\n}";
    return fs.writeFile(path.join(app, '.cozy_conf.json'), data, function(err) {
      if (err) {
        return console.log(err.red);
      } else {
        console.log("Config file successfully saved.".green);
        return callback();
      }
    });
  };

  return RepoManager;

})();
