// Generated by CoffeeScript 1.6.3
var Client, exec, fs, helpers, path;

require('colors');

path = require('path');

fs = require('fs');

Client = require('request-json').JsonClient;

exec = require('child_process').exec;

helpers = require('./helpers');

exports.VagrantManager = (function() {
  function VagrantManager() {
    this.baseBoxURL = 'http://files.cozycloud.cc/cozycloud-dev-latest.box';
    this.docURL = "http://cozy.io/hack/getting-started/setup-environment.html";
  }

  VagrantManager.prototype.checkIfVagrantIsInstalled = function(callback) {
    var _this = this;
    return exec("vagrant -v", function(err, stdout, stderr) {
      var msg;
      if (err) {
        msg = "Vagrant is required to use a virtual machine. " + ("Please, refer to our documentation on " + _this.docURL);
        return console.log(msg.red);
      } else {
        if (callback != null) {
          return callback();
        }
      }
    });
  };

  VagrantManager.prototype.vagrantBoxAdd = function(callback) {
    var cmds;
    cmds = [];
    cmds.push({
      name: 'vagrant',
      args: ['box', 'add', 'cozycloud-dev-latest', this.baseBoxURL]
    });
    return helpers.spawnUntilEmpty(cmds, function() {
      var msg;
      msg = "The base box has been added to your environment or is " + "already installed.";
      console.log(msg.green);
      if (callback != null) {
        return callback();
      }
    });
  };

  VagrantManager.prototype.vagrantInit = function(callback) {
    var cmds,
      _this = this;
    cmds = [];
    cmds.push({
      name: 'vagrant',
      args: ['init', "cozy-dev-latest"]
    });
    return helpers.spawnUntilEmpty(cmds, function() {
      return _this.importVagrantFile(callback);
    });
  };

  VagrantManager.prototype.vagrantBoxDestroy = function(callback) {
    var cmds;
    cmds = [];
    cmds.push({
      name: 'vagrant',
      args: ['--force', 'destroy']
    });
    cmds.push({
      name: 'vagrant',
      args: ['box', 'remove', 'cozycloud-dev-latest']
    });
    cmds.push({
      name: 'rm',
      args: ['-rf', 'Vagrantfile']
    });
    return helpers.spawnUntilEmpty(cmds, callback);
  };

  VagrantManager.prototype.vagrantUp = function(callback) {
    var cmds;
    cmds = [];
    cmds.push({
      name: 'vagrant',
      args: ['up']
    });
    return helpers.spawnUntilEmpty(cmds, callback);
  };

  VagrantManager.prototype.vagrantHalt = function(callback) {
    var cmds;
    cmds = [];
    cmds.push({
      name: 'vagrant',
      args: ['halt']
    });
    return helpers.spawnUntilEmpty(cmds, callback);
  };

  VagrantManager.prototype.vagrantSuspend = function(callback) {
    var cmds;
    cmds = [];
    cmds.push({
      name: 'vagrant',
      args: ['suspend']
    });
    return helpers.spawnUntilEmpty(cmds, callback);
  };

  VagrantManager.prototype.lightUpdate = function(callback) {
    var cmds, scriptUrl;
    console.log("Patching the updater and updating the VM..." + "This may take a while...");
    cmds = [];
    cmds.push({
      name: 'vagrant',
      args: ['ssh', '-c', 'rm -rf ~/update-devenv.sh']
    });
    scriptUrl = "https://raw.github.com/mycozycloud/cozy-setup/master/" + "dev/update-devenv.sh";
    cmds.push({
      name: 'vagrant',
      args: ['ssh', '-c', 'curl -Of ' + scriptUrl]
    });
    cmds.push({
      name: 'vagrant',
      args: ['ssh', '-c', 'chmod u+x ~/update-devenv.sh']
    });
    cmds.push({
      name: 'vagrant',
      args: ['ssh', '-c', '~/update-devenv.sh']
    });
    return this.importVagrantFile(function() {
      return helpers.spawnUntilEmpty(cmds, callback);
    });
  };

  VagrantManager.prototype.importVagrantFile = function(callback) {
    var client, url;
    console.log("Importing latest Vagrantfile version...");
    url = "mycozycloud/cozy-setup/master/dev/Vagrantfile";
    client = new Client("https://raw.github.com/");
    return client.saveFile(url, './Vagrantfile', function(err, res, body) {
      var msg;
      if (err) {
        msg = "An error occurrend while retrieving the Vagrantfile.";
        console.log(msg.red);
      } else {
        console.log("Vagrantfile successfully upgraded.".green);
      }
      if (callback != null) {
        return callback();
      }
    });
  };

  VagrantManager.prototype.virtualMachineStatus = function(callback) {
    var client, url;
    url = "http://localhost:9104";
    console.log("Checking status on " + url + "...");
    client = new Client(url);
    return client.get('/status', function(err, res, body) {
      var app, formattedStatus, isOkay, status;
      if (err) {
        return callback(1);
      } else {
        isOkay = 0;
        for (app in body) {
          status = body[app];
          if (status === true) {
            formattedStatus = "ok".green;
          } else {
            formattedStatus = "ko".red;
            if (app !== "registered") {
              isOkay = 1;
            }
          }
          console.log("\t* " + app + ": " + formattedStatus);
        }
        return callback(isOkay);
      }
    });
  };

  return VagrantManager;

})();
