#!/usr/bin/env node

var sys = require('util');
var fs = require('fs');

try {
    var package = require(process.cwd() + '/package.json');
    if (package.main.match(/\.coffee$/)) {
        require('coffee-script');
    }
    instantiateApp = require(process.cwd());
} catch(e) {
    instantiateApp = null;
}

var app, compound;
if (typeof instantiateApp === 'function') {
    app = instantiateApp();
    compound = app.compound;
}
if (!compound) {
    var Compound = require('compound').Compound;
    compound = new Compound;
}

var args = process.argv.slice(2);
var exitAfterAction = true;
var command = args.shift();

process.nextTick(function () {
  switch (command) {
  default:
  case 'h':
  case 'help':
      if (command && command !== 'help' && command !== 'h') {
          var found = false;
          Object.keys(compound.tools).forEach(function (cmd) {
              var c = compound.tools[cmd];
              if (cmd === command || (c && c.help && c.help.shortcut === command)) {
                  exitAfterAction = c(compound, args);
                  found = true;
              }
          });

          if (found) {
              break;
          }
      }
      var help = [
          'Usage: compound command [argument(s)]\n',
          'Commands:'
      ];
      var commands = [
          ['h', 'help',            'Display usage information'],
          ['i', 'init',            'Initialize compound app'],
          ['g', 'generate [smth]', 'Generate something awesome']
      ];
      Object.keys(compound.tools).forEach(function (cmd) {
          var h = compound.tools[cmd].help;
          if (h && h.usage) {
              commands.push([h.shortcut || '', h.usage, h.description]);
          }
      });
      var maxLen = 0, addSpaces = compound.utils.addSpaces;
      commands.forEach(function (cmd) {
          if (cmd[1].length > maxLen) {
              maxLen = cmd[1].length;
          }
      });
      commands.forEach(function (cmd) {
          help.push('  ' + addSpaces(cmd[0] + ',', 4) + addSpaces(cmd[1], maxLen + 1) + cmd[2]);
      });
      compound.generators.init(compound, args);
      help.push('\nAvailable generators:\n');
      help.push('  ' + compound.generators.list());
      sys.puts(help.join('\n'));
      break;

  case 'i':
  case 'init':
      compound.generators.init(compound);
      compound.generators.perform('app', args);
      break;
  case 'g':
  case 'generate':
      var what = args.shift();
      compound.generators.init(compound);
      if (typeof what == "undefined" || what == null) {
          console.log('Generator not specified, available generators: ', compound.generators.list());
      } else {
          exitAfterAction = !compound.generators.perform(what, args);
      }
      break;
  case '--version':
      console.log(compound.version);
      break;
  }

  if (exitAfterAction) {
      process.exit(0);
  }

});

/*vim ft:javascript*/
