// Generated by CoffeeScript 1.3.3
(function() {
  var Client;

  Client = require("request-json").JsonClient;

  exports.initialize = function(schema, callback) {
    this.schema = schema;
    if (schema.settings.url == null) {
      schema.settings.url = "http://localhost:9101/";
    }
    schema.adapter = new exports.CozyDataSystem(schema);
    return process.nextTick(callback);
  };

  exports.CozyDataSystem = (function() {

    function CozyDataSystem(schema) {
      this.schema = schema;
      this._models = {};
      this.client = new Client(schema.settings.url);
    }

    CozyDataSystem.prototype.define = function(descr) {
      var _this = this;
      this._models[descr.model.modelName] = descr;
      descr.model.search = function(query, callback) {
        return _this.search(descr.model.modelName, query, callback);
      };
      descr.model.defineRequest = function(name, map, callback) {
        return _this.defineRequest(descr.model.modelName, name, map, callback);
      };
      descr.model.request = function(name, params, callback) {
        return _this.request(descr.model.modelName, name, params, callback);
      };
      descr.model.rawRequest = function(name, params, callback) {
        return _this.rawRequest(descr.model.modelName, name, params, callback);
      };
      descr.model.removeRequest = function(name, callback) {
        return _this.removeRequest(descr.model.modelName, name, callback);
      };
      descr.model.requestDestroy = function(name, params, callback) {
        return _this.requestDestroy(descr.model.modelName, name, params, callback);
      };
      descr.model.all = function(params, callback) {
        return _this.all(descr.model.modelName, params, callback);
      };
      descr.model.destroyAll = function(params, callback) {
        return _this.destroyAll(descr.model.modelName, params, callback);
      };
      descr.model.applyRequest = function(params, callback) {
        return _this.applyRequest(descr.model.modelName, params, callback);
      };
      descr.model._forDB = function(data) {
        return _this._forDB(descr.model.modelName, data);
      };
      descr.model.prototype.index = function(fields, callback) {
        return this._adapter().index(this, fields, callback);
      };
      descr.model.prototype.attachFile = function(path, data, callback) {
        return this._adapter().attachFile(this, path, data, callback);
      };
      descr.model.prototype.getFile = function(path, callback) {
        return this._adapter().getFile(this, path, callback);
      };
      descr.model.prototype.saveFile = function(path, filePath, callback) {
        return this._adapter().saveFile(this, path, filePath, callback);
      };
      descr.model.prototype.removeFile = function(path, callback) {
        return this._adapter().removeFile(this, path, callback);
      };
      descr.model.prototype.createAccount = function(data, callback) {
        return this._adapter().createAccount(this, data, callback);
      };
      descr.model.prototype.getAccount = function(callback) {
        return this._adapter().getAccount(this, callback);
      };
      descr.model.prototype.updateAccount = function(data, callback) {
        return this._adapter().updateAccount(this, data, callback);
      };
      descr.model.prototype.mergeAccount = function(data, callback) {
        return this._adapter().mergeAccount(this, data, callback);
      };
      return descr.model.prototype.destroyAccount = function(callback) {
        return this._adapter().destroyAccount(this, callback);
      };
    };

    CozyDataSystem.prototype.exists = function(model, id, callback) {
      var _this = this;
      return this.client.get("data/exist/" + id + "/", function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (!(body != null) || !(body.exist != null)) {
          return callback(new Error("Data system returned invalid data."));
        } else {
          return callback(null, body.exist);
        }
      });
    };

    CozyDataSystem.prototype.find = function(model, id, callback) {
      var _this = this;
      return this.client.get("data/" + id + "/", function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode === 404) {
          return callback(null, null);
        } else if (body.docType !== model) {
          return callback(null, null);
        } else {
          return callback(null, new _this._models[model].model(body));
        }
      });
    };

    CozyDataSystem.prototype.create = function(model, data, callback) {
      var path,
        _this = this;
      path = "data/";
      if (data.id != null) {
        path += "" + data.id + "/";
        delete data.id;
      }
      data.docType = model;
      return this.client.post(path, data, function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode === 409) {
          return callback(new Error("This document already exists"));
        } else if (response.statusCode !== 201) {
          return callback(new Error("Server error occured."));
        } else {
          return callback(null, body._id);
        }
      });
    };

    CozyDataSystem.prototype.save = function(model, data, callback) {
      var _this = this;
      data.docType = model;
      return this.client.put("data/" + data.id + "/", data, function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode === 404) {
          return callback(new Error("Document not found"));
        } else if (response.statusCode !== 200) {
          return callback(new Error("Server error occured."));
        } else {
          return callback();
        }
      });
    };

    CozyDataSystem.prototype.updateAttributes = function(model, id, data, callback) {
      var _this = this;
      return this.client.put("data/merge/" + id + "/", data, function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode === 404) {
          return callback(new Error("Document not found"));
        } else if (response.statusCode !== 200) {
          return callback(new Error("Server error occured."));
        } else {
          return callback();
        }
      });
    };

    CozyDataSystem.prototype.updateOrCreate = function(model, data, callback) {
      var _this = this;
      data.docType = model;
      return this.client.put("data/upsert/" + data.id + "/", data, function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode !== 200 && response.statusCode !== 201) {
          return callback(new Error("Server error occured."));
        } else if (response.statusCode === 200) {
          return callback(null);
        } else if (response.statusCode === 201) {
          return callback(null, body._id);
        }
      });
    };

    CozyDataSystem.prototype.destroy = function(model, id, callback) {
      var _this = this;
      return this.client.del("data/" + id + "/", function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode === 404) {
          return callback(new Error("Document not found"));
        } else if (response.statusCode !== 204) {
          return callback(new Error("Server error occured."));
        } else {
          return callback();
        }
      });
    };

    CozyDataSystem.prototype.index = function(model, fields, callback) {
      var data,
        _this = this;
      data = {
        fields: fields
      };
      return this.client.post("data/index/" + model.id, data, function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode !== 200) {
          return callback(new Error(body));
        } else {
          return callback(null);
        }
      });
    };

    CozyDataSystem.prototype.search = function(model, query, callback) {
      var data,
        _this = this;
      data = {
        query: query
      };
      return this.client.post("data/search/" + (model.toLowerCase()), data, function(error, response, body) {
        var doc, results, _i, _len, _ref;
        if (error) {
          return callback(error);
        } else if (response.statusCode !== 200) {
          return callback(new Error(body));
        } else {
          results = [];
          _ref = body.rows;
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            doc = _ref[_i];
            results.push(new _this._models[model].model(doc));
            if (doc._id != null) {
              doc.id = doc._id;
            }
          }
          return callback(null, results);
        }
      });
    };

    CozyDataSystem.prototype.attachFile = function(model, path, data, callback) {
      var urlPath,
        _this = this;
      if (typeof data === "function") {
        callback = data;
        data = null;
      }
      urlPath = "data/" + model.id + "/attachments/";
      return this.client.sendFile(urlPath, path, data, function(error, response, body) {
        return _this.checkError(error, response, body, 201, callback);
      });
    };

    CozyDataSystem.prototype.getFile = function(model, path, callback) {
      var urlPath,
        _this = this;
      urlPath = "data/" + model.id + "/attachments/" + path;
      return this.client.get(urlPath, function(error, response, body) {
        return _this.checkError(error, response, body, 200, callback);
      });
    };

    CozyDataSystem.prototype.saveFile = function(model, path, filePath, callback) {
      var urlPath,
        _this = this;
      urlPath = "data/" + model.id + "/attachments/" + path;
      return this.client.saveFile(urlPath, filePath, function(error, response, body) {
        return _this.checkError(error, response, body, 200, callback);
      });
    };

    CozyDataSystem.prototype.removeFile = function(model, path, callback) {
      var urlPath,
        _this = this;
      urlPath = "data/" + model.id + "/attachments/" + path;
      return this.client.del(urlPath, function(error, response, body) {
        return _this.checkError(error, response, body, 204, callback);
      });
    };

    CozyDataSystem.prototype.checkError = function(error, response, body, code, callback) {
      if (error) {
        return callback(error);
      } else if (response.statusCode !== code) {
        return callback(new Error(body));
      } else {
        return callback(null);
      }
    };

    CozyDataSystem.prototype.defineRequest = function(model, name, request, callback) {
      var map, path, reduce, view,
        _this = this;
      if (typeof request === "function") {
        map = request;
      } else {
        map = request.map;
        reduce = request.reduce.toString();
      }
      view = {
        reduce: reduce,
        map: "function (doc) {\n  if (doc.docType === \"" + model + "\") {\n    filter = " + (map.toString()) + ";\n    filter(doc);\n  }\n}"
      };
      path = "request/" + (model.toLowerCase()) + "/" + (name.toLowerCase()) + "/";
      return this.client.put(path, view, function(error, response, body) {
        return _this.checkError(error, response, body, 200, callback);
      });
    };

    CozyDataSystem.prototype.request = function(model, name, params, callback) {
      var path,
        _this = this;
      if (typeof params === "function") {
        callback = params;
      }
      path = "request/" + (model.toLowerCase()) + "/" + (name.toLowerCase()) + "/";
      return this.client.post(path, params, function(error, response, body) {
        var doc, results, _i, _len;
        if (error) {
          return callback(error);
        } else if (response.statusCode !== 200) {
          return callback(new Error(body));
        } else {
          results = [];
          for (_i = 0, _len = body.length; _i < _len; _i++) {
            doc = body[_i];
            doc.value.id = doc.value._id;
            results.push(new _this._models[model].model(doc.value));
          }
          return callback(null, results);
        }
      });
    };

    CozyDataSystem.prototype.rawRequest = function(model, name, params, callback) {
      var path,
        _this = this;
      if (typeof params === "function") {
        callback = params;
      }
      path = "request/" + (model.toLowerCase()) + "/" + (name.toLowerCase()) + "/";
      return this.client.post(path, params, function(error, response, body) {
        if (error) {
          return callback(error);
        } else if (response.statusCode !== 200) {
          return callback(new Error(body));
        } else {
          return callback(null, body);
        }
      });
    };

    CozyDataSystem.prototype.removeRequest = function(model, name, callback) {
      var path,
        _this = this;
      path = "request/" + (model.toLowerCase()) + "/" + (name.toLowerCase()) + "/";
      return this.client.del(path, function(error, response, body) {
        return _this.checkError(error, response, body, 204, callback);
      });
    };

    CozyDataSystem.prototype.requestDestroy = function(model, name, params, callback) {
      var path,
        _this = this;
      if (typeof params === "function") {
        callback = params;
      }
      path = "request/" + (model.toLowerCase()) + "/" + (name.toLowerCase()) + "/destroy/";
      return this.client.put(path, params, function(error, response, body) {
        return _this.checkError(error, response, body, 204, callback);
      });
    };

    CozyDataSystem.prototype.all = function(model, params, callback) {
      var view;
      view = "all";
      if ((params != null ? params.view : void 0) != null) {
        view = params.view;
        delete params.view;
      }
      return this.request(model, view, params, callback);
    };

    CozyDataSystem.prototype.destroyAll = function(model, params, callback) {
      var view;
      view = "all";
      if ((params != null ? params.view : void 0) != null) {
        view = params.view;
        delete params.view;
      }
      return this.requestDestroy(model, view, params, callback);
    };

    CozyDataSystem.prototype._forDB = function(model, data) {
      var res,
        _this = this;
      res = {};
      Object.keys(data).forEach(function(propName) {
        if (_this.whatTypeName(model, propName) === 'JSON') {
          return res[propName] = JSON.stringify(data[propName]);
        } else {
          return res[propName] = data[propName];
        }
      });
      return res;
    };

    CozyDataSystem.prototype.whatTypeName = function(model, propName) {
      var ds;
      ds = this.schema.definitions[model];
      return ds.properties[propName] && ds.properties[propName].type.name;
    };

    CozyDataSystem.prototype.getAccount = function(model, callback) {
      var _this = this;
      return this.exists(model, model.id, function(err, res) {
        if (err) {
          return callback(err);
        } else if (!res) {
          return callback(new Error("The linked model doesn't exist"));
        } else {
          if (!(model.account != null)) {
            return callback(new Error("The model doesn't have an account"));
          } else {
            return _this.client.get("account/" + model.account + "/", function(err, res, body) {
              if (err) {
                return callback(err);
              } else if (res.statusCode === 404) {
                return callback(new Error("The account desn't exist"));
              } else if (res.statusCode === 402) {
                return callback(new Error("Data are corrupted"));
              } else if (body.docType !== "Account") {
                return callback(new Error("The document isn't an account"));
              } else {
                return callback(null, body);
              }
            });
          }
        }
      });
    };

    CozyDataSystem.prototype.createAccount = function(model, data, callback) {
      var _this = this;
      return this.exists(model, model.id, function(err, res) {
        if (err) {
          return callback(err);
        } else if (!res) {
          return callback(new Error("The linked model doesn't exist"));
        } else {
          if (model.account != null) {
            return callback(new Error("The model has already an account"));
          } else {
            data = {
              login: data.login,
              password: data.password
            };
            return _this.client.post('account/', data, function(err, res, body) {
              if (err) {
                return callback(err);
              } else if (res.statusCode === 401) {
                return callback(new Error("The account doesn't have a field                                'password'"));
              } else if (res.statusCode !== 201) {
                return callback(new Error("Server error occured."));
              } else {
                data = {
                  account: body._id
                };
                return _this.updateAttributes(model, model.id, data, function(err) {
                  if (err) {
                    return callback(err);
                  } else {
                    model.account = body._id;
                    data._id = body._id;
                    return callback(null, data);
                  }
                });
              }
            });
          }
        }
      });
    };

    CozyDataSystem.prototype.updateAccount = function(model, data, callback) {
      var _this = this;
      return this.exists(model, model.id, function(err, res) {
        if (err) {
          return callback(err);
        } else if (!res) {
          return callback(new Error("The linked model doesn't exist"));
        } else {
          if (!(model.account != null)) {
            return callback(new Error("The model doesn't have an account"));
          } else {
            data = {
              login: data.login,
              password: data.password
            };
            return _this.client.put("account/" + model.account + "/", data, function(err, res, body) {
              if (err) {
                return callback(err);
              } else if (res.statusCode === 404) {
                return callback(new Error("Document not found"));
              } else if (res.statusCode === 401) {
                return callback(new Error("The account doesn't have a field                             'password'"));
              } else if (res.statusCode !== 200) {
                return callback(new Error("Server error occured."));
              } else {
                return callback();
              }
            });
          }
        }
      });
    };

    CozyDataSystem.prototype.mergeAccount = function(model, data, callback) {
      var _this = this;
      return this.exists(model, model.id, function(err, res) {
        if (err) {
          return callback(err);
        } else if (!res) {
          return callback(new Error("The linked model doesn't exist"));
        } else {
          if (!(model.account != null)) {
            return callback(new Error("The model doesn't have an account"));
          } else {
            return _this.client.put("account/merge/" + model.account + "/", data, function(err, res, body) {
              if (err) {
                return callback(err);
              } else if (res.statusCode === 404) {
                return callback(new Error("Document not found"));
              } else if (res.statusCode !== 200) {
                return callback(new Error("Server error occured."));
              } else {
                return callback();
              }
            });
          }
        }
      });
    };

    CozyDataSystem.prototype.destroyAccount = function(model, callback) {
      var _this = this;
      return this.exists(model, model.id, function(err, res) {
        if (err) {
          return callback(err);
        } else if (!res) {
          return callback(new Error("The linked model doesn't exist"));
        } else {
          if (!(model.account != null)) {
            return callback(new Error("The model doesn't have an account"));
          } else {
            return _this.client.del("account/" + model.account + "/", function(err, res, body) {
              var data;
              if (err) {
                return callback(err);
              } else if (res.statusCode === 404) {
                return callback(new Error("Document not found"));
              } else if (res.statusCode !== 204) {
                return callback(new Error("Server error occured."));
              } else {
                data = {
                  account: null
                };
                return _this.updateAttributes(model, model.id, data, function(err) {
                  if (err) {
                    return callback(err);
                  } else {
                    model.account = null;
                    return callback(null);
                  }
                });
              }
            });
          }
        }
      });
    };

    return CozyDataSystem;

  })();

}).call(this);
